Attribute VB_Name = "modGraphicOps"
'******************************************************************'
'*                                                                *'
'*                      TurboCAD for Windows                      *'
'*                   Copyright (c) 1993 - 2001                    *'
'*             International Microcomputer Software, Inc.         *'
'*                            (IMSI)                              *'
'*                      All rights reserved.                      *'
'*                                                                *'
'******************************************************************'

Option Explicit

Sub GraphicDeleteEx()
    Dim gr As Graphic
    
    'A very short-lived graphic
    Set gr = ActiveDrawing.Graphics.Add
    gr.Delete
    
    'Any of these calls will throw an error
    'Debug.Print "Deleted?  " & gr.Deleted
    'gr.Vertices.Add 1, 2, 3
    
    'It's good practice to clear the variable reference
    Set gr = Nothing
End Sub

Sub GraphicCopyEx()
    Dim gr0 As Graphic
    Dim gr As Graphic
    Dim i As Integer
    Dim x As Double
    Dim y As Double
    Dim mtx As Double
    
    'Make a circle centered at 1, 0, 0
    Set gr0 = ActiveDrawing.Graphics.AddCircleCenterAndPoint(1, 0, 0, 0, 0, 0)
    
    For i = 1 To 5
        'Clone original
        Set gr = gr0.Duplicate

        'Move to clone to point on the unit circle
        x = Cos(i * 3.14159 / 3)
        y = Sin(i * 3.14159 / 3)
        Set mtx = gr.MoveAbsolute(x, y, 0)
    Next i
    
    'Modify original -- clones are not affected
    With gr0.Properties
        .Item("BrushStyle") = "Solid"
        .Item("BrushColor") = NamedColors("Red")
    End With
End Sub


Sub GraphicRotateAxisEx()
    Dim gr0 As Graphic
    Dim gr As Graphic
    Dim i As Integer
    Dim mtx As Matrix

    Set gr0 = ActiveDrawing.Graphics(0)
    For i = 1 To 5
        Set gr = gr0.Duplicate
        Set mtx = gr.RotateAxis(i * 3.14159 / 6)
    Next
    
    'Modify original -- clones are not affected
    With gr0.Properties
        .Item("BrushStyle") = "Solid"
        .Item("BrushColor") = NamedColors("Red")
    End With
End Sub

Sub GraphicRotateAbsEx()
    Dim gr0 As Graphic
    Dim gr As Graphic
    Dim mtx As Matrix

    Set gr0 = ActiveDrawing.Graphics(0)
    Set gr = gr0.Duplicate
    Set mtx = gr.RotateAbsolute(1, 1, 0, 0, 0, 0)
    
End Sub

Sub GraphicScaleEx()
    Dim gr0 As Graphic
    Dim gr As Graphic
    Dim i As Integer
    Dim mtx As Matrix

    Set gr0 = ActiveDrawing.Graphics(0)
    Set gr = gr0.Duplicate
    Set mtx = gr.Scale(1, 2, 1)
    Set gr = gr0.Duplicate
    Set mtx = gr.Scale(2, 1, 1)
    Set gr = gr0.Duplicate
    Set mtx = gr.Scale(-1, 2, 1)
    Set gr = gr0.Duplicate
    Set mtx = gr.Scale(-2, 1, 1)
    Set gr = gr0.Duplicate
    Set mtx = gr.Scale(-1, -2, 1)
    Set gr = gr0.Duplicate
    Set mtx = gr.Scale(-2, -1, 1)
    Set gr = gr0.Duplicate
    Set mtx = gr.Scale(1, -2, 1)
    Set gr = gr0.Duplicate
    Set mtx = gr.Scale(2, -1, 1)

    'Modify original -- clones are not affected
    With gr0.Properties
        .Item("BrushStyle") = "Solid"
        .Item("BrushColor") = NamedColors("Red")
    End With
End Sub

Sub GraphicTransformEx()
    Dim gr0 As Graphic
    Dim gr As Graphic
    Dim mtx As Matrix
    
    Set mtx = New XMatrix
    
    'Set up a pure translation matrix
    'Translation (5, 6, 7)
    mtx.SetEntries _
        1, 0, 0, 0, _
        0, 1, 0, 0, _
        0, 0, 0, 0, _
        5, 6, 7, 1
    
    Set gr0 = ActiveDrawing.Graphics(0)
    Set gr = gr0.Duplicate
    gr.Transform mtx
End Sub

Sub GraphicUCS()
    Dim gr As Graphic
    Dim mtx(0 To 15) As Double
    Dim i As Integer
    
    For Each gr In ActiveDrawing.Graphics
        gr.UCS.GetEntries _
            mtx(0), mtx(1), mtx(2), mtx(3), _
            mtx(4), mtx(5), mtx(6), mtx(7), _
            mtx(8), mtx(9), mtx(10), mtx(11), _
            mtx(12), mtx(13), mtx(14), mtx(15)

        Debug.Print "Matrix"
        Debug.Print mtx(0) & " " & mtx(1) & " " & mtx(2) & " " & mtx(3)
        Debug.Print mtx(4) & " " & mtx(5) & " " & mtx(6) & " " & mtx(7)
        Debug.Print mtx(8) & " " & mtx(9) & " " & mtx(10) & " " & mtx(11)
        Debug.Print mtx(12) & " " & mtx(13) & " " & mtx(14) & " " & mtx(15)
    Next
End Sub
